# This file is adapted from python code released by WellDone International
# under the terms of the LGPLv3.  WellDone International's contact information is
# info@welldone.org
# http://welldone.org
#
# Modifications to this file from the original created at WellDone International 
# are copyright Arch Systems Inc.

#basic tests to ensure that everything is installed and working properly

def test_required_packages():
	"""
	Make sure that the required dependencies of iotile.core are installed
	"""

	import decorator
	#TODO add the rest of the packages

def test_import_iotilecore():
	"""
	Make sure iotile.core is accessible for import
	"""

	import iotile.core